import { useEffect, useState, createContext, useMemo } from 'react'
import { useRouter } from 'next/router'
import { execute } from '@/helper/clientApi'
import environment from '@/utils/environment'
import presets from '@/utils/globalPresets'
import dynamic from 'next/dynamic'
import useLoading from '@/hooks/useLoading'
import { useStore } from '@/hooks/useStore'
import useHasMounted from '@/hooks/useHasMounted'
import useI18n from '@/hooks/useI18n'
import SessionTimeout from '@/components/SessionTimeout'
import UserOptionsMenu from '@/components/widgets/UserOptionsMenu'
import { toast } from 'react-toastify'

export const LayoutContext = createContext()

const Footer = dynamic(() => { return import('vComponents/dist/Footer') }, { ssr: false })
const Sidebar = dynamic(() => { return import('vComponents/dist/Sidebar') }, { ssr: false })
const Navbar = dynamic(() => { return import('vComponents/dist/Navbar') }, { ssr: false })

const ResponsiveContainer = ({ children }) => {
  const router = useRouter()
  const loading = useLoading()
  const hasMounted = useHasMounted()
  const i18n = useI18n()
  const [_i18n, setI18n] = useState({ t: () => { return 'txtNone' } })

  const [sidebarOpen, setSidebarOpen] = useState(true)
  const [menu, setMenu] = useState([])
  const [title, setTitle] = useStore(s => s.title, a => a.setTitle)
  const [userObj, setUserObj] = useState()
  const [token, setToken] = useState('')
  const [theme, setTheme] = useState(null)
  const [constanteObj, setConstanteObj] = useState('')

  const doLogout = async () => {
    const redirectPath = await environment.logout()
    await i18n.locale('es')
    setUserObj(null)
    setConstanteObj(null)
    setTheme(null)
    router.push(redirectPath)
  }

  // se ejecuta cuando expira la sesion
  const onTimeout = () => {
    doLogout()
  }

  // se ejecuta cuando el usuario indica explicitamente que sigue conectado en la sesion
  const onCancelTimeout = async () => {
    const sprUpdateSesion = 'SPR_SESION_U'
    const result = await execute(sprUpdateSesion, [token])
    if (environment.validaResultadoDB(result) === true) {
      const updateUserObj = { ...userObj }
      updateUserObj.tiempo_expiracion = result[0].tiempo_expiracion
      updateUserObj.fecha_fin_sesion = result[0].fecha_fin_sesion
      setUserObj(updateUserObj)
      await environment.login(token, updateUserObj)
    }
  }

  const setPreferences = async (opcion) => {
    const env = await environment.getEnvUser()
    // console.log((new Date()).toString(),'setPreferences userObj', userObj)
    const sprInfoSesion = 'SPR_INFO_USUARIOS'
    const response3 = await execute(sprInfoSesion, [env.token, opcion])
    if (environment.validaResultadoDB(response3, i18n, toast, false) === true) {
      // console.log('setPreferences', response3)
      if (opcion === 'en' || opcion === 'es') {
        env.user.i18n = opcion
        // setLanguage(opcion)
      }
      if (opcion === 'light' || opcion === 'dark') {
        env.user.theme = opcion
        setTheme(opcion)
      }
      environment.setEnvUser(env.user)
      if (environment.validaResultadoDB(response3, i18n, toast, false) === true) {
        if (response3 && response3[0]) {
          toast.success(response3[0].response, presets.toaster)
        }
      }
    }
  }

  const onBackKeyDown = () => {
    if (router.asPath === '/') {
      // cuando esta en la pagina principal cierra la app
      if (navigator.app) {
        navigator.app.exitApp()
      } else if (navigator.device) {
        navigator.device.exitApp()
      } else {
        window.close()
      }
    } else {
      window.history.back()
    }
  }

  const validaPermisosRuta = async (token) => {
    if (router.pathname.includes('/admin')) {
      loading.start()
      const tienePermiso = await environment.validaPermisos(token, 'S', 'N', 'N', 'N')
      loading.stop()
      if (tienePermiso === false) {
        router.push(presets.locations.welcome)
      }
    }
    if (router.pathname.includes('/mantenimientos')) {
      loading.start()
      const tienePermiso = await environment.validaPermisos(token, 'N', 'S', 'N', 'N')
      loading.stop()
      if (tienePermiso === false) {
        router.push(presets.locations.welcome)
      }
    }
  }

  const getMenu = async () => {
    loading.start()
    setMenu([])
    if (token && token !== null) {
      const options = await execute('SPR_MENU_S', [token, 'BO', null])
      setMenu(options)
    }
    loading.stop()
  }

  const getEnv = async () => {
    try {
      const env = await environment.getEnvUser()

      if (!env || !env.token) {
        doLogout()
        return
      }
      setUserObj(env.user)
      setConstanteObj(env.constante)
      setToken(env.token)
      setTheme(env.user.theme || 'light')
      if (!env.user.i18n) {
        const browserLanguage = window.navigator.language
        if (browserLanguage === 'en' || browserLanguage === 'en-US') {
          setPreferences('en')
        } else {
          setPreferences('es')
        }
      } else {
        setPreferences(env.user.idioma)
      }
    } catch (error) {
      doLogout()
    }
  }

  const getTheme = () => {
    if (!theme || theme != null) {
      return theme
    }
    return 'light'
  }

  const containerWrapper = useMemo(() => ({
    userObj,
    setUserObj,
    constanteObj,
    setConstanteObj,
    token,
    setToken,
    setTheme,
    setTitle
  }), [userObj, setUserObj, constanteObj, setConstanteObj, token, setToken, setTheme, setTitle])

  useEffect(() => {
    setI18n(i18n)
  }, [i18n])

  useEffect(() => {
    getMenu()
  }, [token])

  // when page is mounted
  useEffect(() => {
    if (hasMounted) {
      getEnv().then(() => {
        getMenu()
      })
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [hasMounted])

  useEffect(() => {
    if (process.browser && window.cordova) {
      document.addEventListener('backbutton', onBackKeyDown, false)
    }
  })

  return (
    <LayoutContext.Provider value={containerWrapper}>

      <div className={`flex h-screen overflow-hidden ${getTheme()}`} >
        {/* Sidebar */}
        {
          userObj && userObj.nombre_usuario &&

          <Sidebar
            sidebarOpen={sidebarOpen}
            setSidebarOpen={setSidebarOpen}
            menu={menu}
            sidebarStyles={'via-bg-sidebar'}
            appTitleStyles='via-apptitle-sidebar'
            userInfoStyles='via-userinfo-sidebar'
            optionStyles={'via-options-sidebar'}
            iconOptionStyles={'via-icons-sidebar'}
            suboptionStyles={'via-suboptions-sidebar'}
            iconSuboptionStyles={'via-icons-subsidebar'}
            onClickLogout={() => doLogout()}
            setTitle={setTitle}
            userObj={userObj}
            environment={environment}
            presets={presets}
            router={router}
            i18n={_i18n}
          />
        }

        {/* Content area */}
        <div className={`${sidebarOpen ? 'relative' : 'absolute'} flex h-screen w-full flex-1 flex-col overflow-y-auto overflow-x-hidden bg-gray-100 dark:bg-gray-700`}>

          {/*  Site header */}
          {userObj && userObj.nombre_usuario &&

            <Navbar
              sidebarOpen={sidebarOpen}
              setSidebarOpen={setSidebarOpen}
              navbarStyles="border-b via-navbar"
              menuButtonStyles="via-menu-btn-navbar"
              userMenuButtonStyles="via-usermenu-btn-navbar"
              userOptionStyles="via-useroptions-navbar"
              userOptionSelectedStyles="via-useroptions-selected-navbar"
              onClickLogout={() => doLogout()}
              onClickProfile={() => router.push(`${presets.locations.profile}/${environment.getTime()}`)}
              MenuOptions={() => <UserOptionsMenu setPreferences={setPreferences} theme={theme} i18n={i18n} />}
              title={title}
              userObj={userObj}
              router={router}
              presets={presets}

            />

          }

          <main>
            <div className={userObj && userObj.nombre_usuario ? 'responsive-container' : ''}>
              {children}
            </div>
          </main>
          {
            userObj && userObj.nombre_usuario &&
            <SessionTimeout i18n={_i18n} user={userObj} onTimeout={() => onTimeout()} onCancelTimeout={() => onCancelTimeout()} />
          }
        </div>

      </div>
      <Footer />
    </LayoutContext.Provider>
  )
}

export default ResponsiveContainer
